@@ -1,4 +1,4 @@
 from .pipeline import (handles_request_head, handles_request_body,
                        handles_response_head, handles_response_body,
                        HttpFilter, HttpFilterPipeline, consume, reject,
-                       route, next)
+                       route, resume)
